#include <em_device.h>
#include "hardware.h"

void CMU_setup(void)									// configure clock
{
	CMU->HFCORECLKEN0 |= CMU_HFCORECLKEN0_LE;			// required to enable RTC
	CMU->OSCENCMD = CMU_OSCENCMD_LFRCOEN;				// enable LFRCO
	CMU->LFCLKSEL = CMU_LFCLKSEL_LFA_LFRCO;				// set LFRCO as LFA
}

void GPIO_setup(void)									// configure GPIO ports
{
	CMU->HFPERCLKEN0 |= CMU_HFPERCLKEN0_GPIO;			// enable clock to GPIO
	(GPIO->P[1]).MODEL = GPIO_P_MODEL_MODE7_INPUT; 		// set input mode on ECHO (PB.7)
	(GPIO->P[1]).MODEH = GPIO_P_MODEH_MODE8_PUSHPULL+ 	// set output mode on PULSE (PB.8) 
						 GPIO_P_MODEH_MODE13_INPUTPULL;	// set pull mode on BUTTON (PB.13)
	(GPIO->P[1]).DOUT = BUTTON;							//   set pullup
	GPIO->EXTIPSELL = GPIO_EXTIPSELL_EXTIPSEL7_PORTB;	// enable ECHO interrupt at PB.7
	GPIO->EXTIPSELH = GPIO_EXTIPSELH_EXTIPSEL13_PORTB;	// enable BUTTON interrupt at PB.13
	GPIO->EXTIFALL = BUTTON + ECHO;						// falling edge interrupts
	GPIO->IEN = BUTTON + ECHO;
	NVIC_EnableIRQ(GPIO_ODD_IRQn);						// enable interrupts in NVIC
}

void PRS_setup(void)
{
	CMU->HFPERCLKEN0 |= CMU_HFPERCLKEN0_PRS;			// enable HF clock to PRS
	(PRS->CH[0]).CTRL = PRS_CH_CTRL_EDSEL_POSEDGE+
						PRS_CH_CTRL_SOURCESEL_GPIOL+ 	// set CH0 to trigger TMR0_CH0/TMR1_CH0 timer reload
						PRS_CH_CTRL_SIGSEL_GPIOPIN7;
	(PRS->CH[1]).CTRL = PRS_CH_CTRL_EDSEL_NEGEDGE+
						PRS_CH_CTRL_SOURCESEL_GPIOL+ 	// set CH1 to trigger TMR0_CH1/TMR1_CH1 capture
						PRS_CH_CTRL_SIGSEL_GPIOPIN7;
}

void Timers_setup(void)
{
	CMU->HFPERCLKEN0 |= CMU_HFPERCLKEN0_TIMER0+ 		// enable clock to timers
						CMU_HFPERCLKEN0_TIMER1;	
	
	TIMER0->CTRL = TIMER_CTRL_PRESC_DIV128+				// run timer at 109375Hz = 14MHz / 128
				   TIMER_CTRL_RISEA_RELOADSTART;
	
	(TIMER0->CC[0]).CTRL = TIMER_CC_CTRL_INSEL_PRS+		// use PRS to zero the timer
						   TIMER_CC_CTRL_PRSSEL_PRSCH0+	//    ... channel 0
						   TIMER_CC_CTRL_MODE_INPUTCAPTURE; //... capture mode
	
	(TIMER0->CC[1]).CTRL = TIMER_CC_CTRL_INSEL_PRS+		// use PRS to capture the measurement
						   TIMER_CC_CTRL_PRSSEL_PRSCH1+	//    ... channel 1
						   TIMER_CC_CTRL_MODE_INPUTCAPTURE; //... capture mode
	TIMER0->CMD = TIMER_CMD_START;						// start timer0

	TIMER1->CTRL = TIMER_CTRL_CLKSEL_TIMEROUF+			// clock is TMR0_OF
				   TIMER_CTRL_RISEA_RELOADSTART;
	
	(TIMER1->CC[0]).CTRL = TIMER_CC_CTRL_INSEL_PRS+		// use PRS to zero the timer
						   TIMER_CC_CTRL_PRSSEL_PRSCH0+	//    ... channel 0
						   TIMER_CC_CTRL_MODE_INPUTCAPTURE; //... capture mode
	
	(TIMER1->CC[1]).CTRL = TIMER_CC_CTRL_INSEL_PRS+  	// use PRS to capture the measurement
						   TIMER_CC_CTRL_PRSSEL_PRSCH1+ //    ... channel 1
						   TIMER_CC_CTRL_MODE_INPUTCAPTURE;
	TIMER1->CMD = TIMER_CMD_START;						// start timer1
}

void I2C0_setup(void)
{
	CMU->HFPERCLKEN0 |= CMU_HFPERCLKEN0_I2C0;			// enable clock to I2C0
	(GPIO->P[3]).MODEL |= GPIO_P_MODEL_MODE6_WIREDAND+	// PD.6/PD.7 = SDA/SCL
						  GPIO_P_MODEL_MODE7_WIREDAND;
	I2C0->ROUTE = I2C_ROUTE_LOCATION_LOC1 +				// enable I2C pins at location 1
				  I2C_ROUTE_SDAPEN + I2C_ROUTE_SCLPEN; 	// SDA/SCL
	I2C0->CLKDIV = 6;									// set 100 KHz I2C clock (@ PERCLK=14 MHz)
	I2C0->CTRL = I2C_CTRL_EN;							// clear module reset
}

void RTC_setup(void)									// configure RTC
{
	CMU->LFACLKEN0 |= CMU_LFACLKEN0_RTC; 				// enable LFA to RTC
	RTC->COMP0 = 32768;									// 1 sec period setup
	RTC->IEN = RTC_IEN_COMP0;
	RTC->CTRL = RTC_CTRL_COMP0TOP + RTC_CTRL_EN;		// start RTC
}

void Delay_ms(uint32_t d)
{
	RTC->COMP0 = 33*d;
	RTC->CTRL = RTC_CTRL_EN;							// start RTC
	while (! (RTC->IF & RTC_IF_COMP0)){}
	RTC->IFC = RTC_IFC_COMP0;							// clear RTC flag
	RTC->CTRL = 0;										// stop RTC		
}
