#include <em_device.h>								// header for all micro registers
#include "hardware.h"								// custom headers
#include "dogs_10x4.h"
#include <stdio.h>
#include <math.h>
#define soundVelocity  34.322						// cm/ms

volatile uint8_t buttonFlag, echoFlag;				// software event flags
uint32_t ticks, distance;							// echo width in timer ticks 
float returnTime;									// distance to object
char buffer[4][11];
int bufIndex;

void toString(char* s, uint32_t d);					// function prototypes

int main()
{
	CMU_setup();									// configure hardware							
	GPIO_setup();							 
	PRS_setup();
	Timers_setup();	
	RTC_setup();	
	I2C0_setup();
	DOGS_setup();

	buffer[0][0] = buffer [1][0] = buffer[2][0] = buffer[3][0] = 0;
	bufIndex = 0;
	buttonFlag = echoFlag = 0;
	DOGS_WriteRow("Ready", DOGS_FIRST_ROW);
	
	while(1)										// main app loop
	{	
		__WFI();
		
		if (buttonFlag)								// distance measurement is requested
		{
			(GPIO->P[1]).DOUTSET = TRIG;			// generate a ~10uSec pulse
			for (uint8_t i=0; i<30; i++){}			// pulse duration delay
			(GPIO->P[1]).DOUTCLR = TRIG;
			buttonFlag = 0;							// reset flag	
		}
		
		if (echoFlag)
		{					
			ticks = ((TIMER1->CC[1]).CCVB << 16) + (TIMER0->CC[1]).CCVB; 	// get timer readings
			returnTime = (ticks*1000.0) / 109375;						// echo time in milliseconds
			distance = (uint32_t)((returnTime * soundVelocity)/2 + 0.5);// distance in cm

			toString(buffer[bufIndex], distance);	// convert distance to string
						
			// Print out the circular buffer to the screen 
			DOGS_WriteRow(buffer[bufIndex], DOGS_FOURTH_ROW);
			DOGS_WriteRow(buffer[(bufIndex - 1) & 3], DOGS_THIRD_ROW);
			DOGS_WriteRow(buffer[(bufIndex - 2) & 3], DOGS_SECOND_ROW);
			DOGS_WriteRow(buffer[(bufIndex - 3) & 3], DOGS_FIRST_ROW);			
			bufIndex = (bufIndex + 1) & 3;								// update circ buffer index
			
			while (!((GPIO->P[1]).DIN & BUTTON)){}	// wait for button release
			Delay_ms(100);
			echoFlag = 0;
			buttonFlag = 0;		
		}	
	}
}

void toString(char* s, uint32_t d)					// convert 32-bit unsigned int to string
{
	uint8_t rem[8];									// storage for digits
	int32_t i,j = 0;
	if (d == 0)
	{
		s[0] = '0';									// handle special case d = 0
		j = 1;
	}
	else
	{
		i=0;
		do											// loop over all digits
		{
			rem[i++] = d % 10;						// compute digit
			d = d / 10;
		} while (d != 0);							// supress leading 0s
		for (i=i-1; i>=0; i--)						// print digits
			s[j++] = rem[i] + '0';
	}
	s[j++] = ' ';
	s[j++] = 'c';
	s[j++] = 'm';
	for (; j<10; j++)								// erase remaining chars in buffer
		s[j] = ' ';							
	s[j] = 0;										// string termination
}

void GPIO_ODD_IRQHandler()							// even pins interrupt handler
{
	if (GPIO->IF & BUTTON)							// check the interrupt source
	{
		GPIO->IFC = BUTTON;							// clear button IF
		buttonFlag = 1;								// set flag
	}	
	if (GPIO->IF & ECHO)
	{
		GPIO->IFC = ECHO;							// clear echo IF
		echoFlag = 1;								// set flag
	}
}

